% Clear workspace and close all figures
% clear all; close all; clc;

% Sampling frequency
Fs = 1 / (150e-6); % Sampling frequency in Hz (150 microseconds sampling interval)

% Define the frequency range and number of bands
freqRange = [200, 1600]; % Define the start and end frequency (in Hz)
numBands = 5;            % Number of bands to divide the frequency range
bandWidth = diff(freqRange) / numBands; % Calculate band width dynamically

%% Process a Single File with Detailed Steps

% File and data range parameters
filename = '/Users/davidperez-cruet/Desktop/data12_8/vibration_data.csv'; % Use only the first file for this test

numRows = 4; % Five rows for different stages of analysis
numCols = 3; % Three columns for raw data, Hann filter, and Tukey filter

% Import the data
dataRange = 'A:B';

rawData = readmatrix(filename, 'Range', dataRange);



timestamps = rawData(:, 1); % First column: timestamps
sensorData = rawData(:, 2); % Second column: sensor values

% Step 1: Detect signal start and end points
windowSize = 100; % Moving average window size
smoothSignal = movmean(abs(sensorData), windowSize); % Smooth signal

% Compute thresholds
baselineDuration = round(0.1 * length(sensorData)); % First 10% of data
baselineValue = mean(smoothSignal(1:baselineDuration)); % Baseline mean
baselineThreshold = baselineValue + 5; % Baseline threshold
activeThreshold = max(smoothSignal) * 0.2; % Dynamic threshold
combinedThreshold = max(baselineThreshold, activeThreshold); % Final threshold

% Identify active signal region
signalIndices = find(smoothSignal > combinedThreshold);
if ~isempty(signalIndices)
    startIdx = signalIndices(1); % Start index
    endIdx = signalIndices(end); % End index
else
    startIdx = NaN; % Handle missing signal
    endIdx = NaN;
end

% Initialize figure for subplots
figure;

% Left Column: Raw Data
% Step 2: Plot raw data with signal start/end detection
subplot(numRows, numCols, 1); % Row 1, Column 1
plot(timestamps, sensorData, 'b'); % Raw data
hold on;
plot(timestamps, smoothSignal, 'r', 'LineWidth', 1); % Overlay smoothed signal
if ~isnan(startIdx) && ~isnan(endIdx)
    plot(timestamps(startIdx), smoothSignal(startIdx), 'go', 'MarkerSize', 8, 'DisplayName', 'Start');
    plot(timestamps(endIdx), smoothSignal(endIdx), 'ro', 'MarkerSize', 8, 'DisplayName', 'End');
end
hold off;
title('Raw Data with Signal Start/End Detection');
xlabel('Time (s)');
ylabel('Amplitude');
grid on;

% Step 3: Extract and plot raw data from start-end region
if ~isnan(startIdx) && ~isnan(endIdx)
    extractedSignal = sensorData(startIdx:endIdx);
    extractedTimestamps = timestamps(startIdx:endIdx);
else
    extractedSignal = [];
    extractedTimestamps = [];
end
subplot(numRows, numCols, 4); % Row 2, Column 1
if ~isempty(extractedSignal)
    plot(extractedTimestamps, extractedSignal);
    title('Extracted Signal');
    xlabel('Time (s)');
    ylabel('Amplitude');
    grid on;
else
    title('No Signal Detected');
end

% Step 4: Compute and plot normalized FFT (raw data)
if ~isempty(extractedSignal)
    N_extracted = length(extractedSignal);
    f_extracted = (0:N_extracted-1) * (Fs / N_extracted);
    fftExtracted = fft(extractedSignal, N_extracted);
    fftExtracted_half = fftExtracted(1:floor(N_extracted / 2));
    f_extracted_half = f_extracted(1:floor(N_extracted / 2));
    fftExtractedMag = abs(fftExtracted_half);
    fftNormalized = fftExtractedMag / sum(fftExtractedMag); % Normalize FFT

    subplot(numRows, numCols, 7); % Row 3, Column 1
    plot(f_extracted_half, fftNormalized, 'k');
    xlim([10, 1600]);
    title('Normalized FFT (Raw Data)');
    xlabel('Frequency (Hz)');
    ylabel('Normalized Magnitude');
    grid on;
end

% Hann Filter Column
if ~isempty(extractedSignal)
    hannWindow = hann(length(extractedSignal));
    windowedSignalHann = extractedSignal .* hannWindow; % Apply Hann window
    subplot(numRows, numCols, 5); % Row 2, Column 2
    plot(extractedTimestamps, windowedSignalHann);
    title('Hann Windowed Signal');
    xlabel('Time (s)');
    ylabel('Amplitude');
    grid on;

    % Compute and plot normalized FFT using the Hann windowed signal
    fftWindowedHann = fft(windowedSignalHann, N_extracted);
    fftWindowedHann_half = fftWindowedHann(1:floor(N_extracted / 2));
    fftWindowedHannMag = abs(fftWindowedHann_half);
    fftNormalizedHann = fftWindowedHannMag / sum(fftWindowedHannMag); % Normalize

    subplot(numRows, numCols, 8); % Row 3, Column 2
    plot(f_extracted_half, fftNormalizedHann, 'k');
    xlim([10, 1600]);
    title('Normalized FFT (Hann)');
    xlabel('Frequency (Hz)');
    ylabel('Normalized Magnitude');
    grid on;
end

% Tukey Filter Column
if ~isempty(extractedSignal)
    tukeyWindow = tukeywin(length(extractedSignal), 0.2); % Tukey window with 20% tapering
    windowedSignalTukey = extractedSignal .* tukeyWindow; % Apply Tukey window
    subplot(numRows, numCols, 6); % Row 2, Column 3
    plot(extractedTimestamps, windowedSignalTukey);
    title('Tukey Windowed Signal');
    xlabel('Time (s)');
    ylabel('Amplitude');
    grid on;

    % Compute and plot normalized FFT using the Tukey windowed signal
    fftWindowedTukey = fft(windowedSignalTukey, N_extracted);
    fftWindowedTukey_half = fftWindowedTukey(1:floor(N_extracted / 2));
    fftWindowedTukeyMag = abs(fftWindowedTukey_half);
    fftNormalizedTukey = fftWindowedTukeyMag / sum(fftWindowedTukeyMag); % Normalize

    subplot(numRows, numCols, 9); % Row 3, Column 3
    plot(f_extracted_half, fftNormalizedTukey, 'k');
    xlim([10, 1600]);
    title('Normalized FFT (Tukey)');
    xlabel('Frequency (Hz)');
    ylabel('Normalized Magnitude');
    grid on;
end
% Final Row: Add smoothed FFTs for all columns and perform peak detection
if ~isempty(fftNormalized)
    % Raw Data
    fftSmoothed = movmean(fftNormalized, 15); % Use same smoothing window
    
    % Peak Detection on Smoothed FFT (Raw)
    minFreq = freqRange(1);
    maxFreq = freqRange(2);
    validIndices = (f_extracted_half >= minFreq & f_extracted_half <= maxFreq);
    freqRangeData = f_extracted_half(validIndices);
    fftSmoothedRange = fftSmoothed(validIndices);
    
    [pks, locs] = findpeaks(fftSmoothedRange, freqRangeData, 'MinPeakDistance', 30);
    if ~isempty(pks)
        [sortedPks, sortIdx] = sort(pks, 'descend');
        numPeaks = min(5, length(sortedPks));
        topIdx = sortIdx(1:numPeaks);
        topPks = sortedPks(1:numPeaks);
        topLocs = locs(topIdx);
        
        % Print out the top peaks
        fprintf('\nTop Peaks (Raw Data):\n');
        for k = 1:numPeaks
            fprintf('Peak %d: Frequency = %.2f Hz, Magnitude = %.4f\n', k, topLocs(k), topPks(k));
        end
    else
        warning('No peaks detected in Raw Data');
    end
    
    % Plot smoothed FFT and peaks
    subplot(numRows, numCols, 10); % Row 4, Column 1
    plot(freqRangeData, fftSmoothedRange, 'b');
    title('Smoothed FFT (Raw)');
    xlabel('Frequency (Hz)');
    ylabel('Magnitude');
    xlim([freqRange(1), freqRange(2)]); % Limit x-axis to freqRange
    grid on;
    hold on;
    if ~isempty(pks)
        plot(topLocs, topPks, 'r*', 'MarkerSize', 8);
    end
    hold off;
    
    % Energy Distribution (Raw Data)
    bandEnergies = zeros(1, numBands);
    fprintf('\nEnergy Distribution Across Frequency Bands (Raw Data):\n');
    for b = 1:numBands
        bandStart = freqRange(1) + (b-1) * bandWidth;
        bandEnd = freqRange(1) + b * bandWidth;
        bandIndices = (freqRangeData >= bandStart & freqRangeData < bandEnd);
        bandEnergies(b) = sum(fftSmoothedRange(bandIndices).^2);
        fprintf('Band %d (%.1f-%.1f Hz): Energy = %.6f\n', b, bandStart, bandEnd, bandEnergies(b));
    end
end

% Hann Data
if exist('fftNormalizedHann', 'var')
    fftSmoothedHann = movmean(fftNormalizedHann, 15);
    
    % Peak Detection on Smoothed FFT (Hann)
    fftSmoothedRange = fftSmoothedHann(validIndices);
    [pks, locs] = findpeaks(fftSmoothedRange, freqRangeData, 'MinPeakDistance', 30);
    if ~isempty(pks)
        [sortedPks, sortIdx] = sort(pks, 'descend');
        numPeaks = min(5, length(sortedPks));
        topIdx = sortIdx(1:numPeaks);
        topPks = sortedPks(1:numPeaks);
        topLocs = locs(topIdx);
        
        % Print out the top peaks
        fprintf('\nTop Peaks (Hann Windowed Data):\n');
        for k = 1:numPeaks
            fprintf('Peak %d: Frequency = %.2f Hz, Magnitude = %.4f\n', k, topLocs(k), topPks(k));
        end
    else
        warning('No peaks detected in Hann Windowed Data');
    end
    
    % Plot smoothed FFT and peaks
    subplot(numRows, numCols, 11); % Row 4, Column 2
    plot(freqRangeData, fftSmoothedRange, 'r');
    title('Smoothed FFT (Hann)');
    xlabel('Frequency (Hz)');
    ylabel('Magnitude');
    xlim([freqRange(1), freqRange(2)]); % Limit x-axis to freqRange
    grid on;
    hold on;
    if ~isempty(pks)
        plot(topLocs, topPks, 'r*', 'MarkerSize', 8);
    end
    hold off;
    
    % Energy Distribution (Hann Data)
    bandEnergies = zeros(1, numBands);
    fprintf('\nEnergy Distribution Across Frequency Bands (Hann Data):\n');
    for b = 1:numBands
        bandStart = freqRange(1) + (b-1) * bandWidth;
        bandEnd = freqRange(1) + b * bandWidth;
        bandIndices = (freqRangeData >= bandStart & freqRangeData < bandEnd);
        bandEnergies(b) = sum(fftSmoothedRange(bandIndices).^2);
        fprintf('Band %d (%.1f-%.1f Hz): Energy = %.6f\n', b, bandStart, bandEnd, bandEnergies(b));
    end
end

% Tukey Data
if exist('fftNormalizedTukey', 'var')
    fftSmoothedTukey = movmean(fftNormalizedTukey, 15);
    
    % Peak Detection on Smoothed FFT (Tukey)
    fftSmoothedRange = fftSmoothedTukey(validIndices);
    [pks, locs] = findpeaks(fftSmoothedRange, freqRangeData, 'MinPeakDistance', 30);
    if ~isempty(pks)
        [sortedPks, sortIdx] = sort(pks, 'descend');
        numPeaks = min(5, length(sortedPks));
        topIdx = sortIdx(1:numPeaks);
        topPks = sortedPks(1:numPeaks);
        topLocs = locs(topIdx);
        
        % Print out the top peaks
        fprintf('\nTop Peaks (Tukey Windowed Data):\n');
        for k = 1:numPeaks
            fprintf('Peak %d: Frequency = %.2f Hz, Magnitude = %.4f\n', k, topLocs(k), topPks(k));
        end
    else
        warning('No peaks detected in Tukey Windowed Data');
    end
    
    % Plot smoothed FFT and peaks
    subplot(numRows, numCols, 12); % Row 4, Column 3
    plot(freqRangeData, fftSmoothedRange, 'g');
    title('Smoothed FFT (Tukey)');
    xlabel('Frequency (Hz)');
    ylabel('Magnitude');
    xlim([freqRange(1), freqRange(2)]); % Limit x-axis to freqRange
    grid on;
    hold on;
    if ~isempty(pks)
        plot(topLocs, topPks, 'r*', 'MarkerSize', 8);
    end
    hold off;
    
    % Energy Distribution (Tukey Data)
    bandEnergies = zeros(1, numBands);
    fprintf('\nEnergy Distribution Across Frequency Bands (Tukey Data):\n');
    for b = 1:numBands
        bandStart = freqRange(1) + (b-1) * bandWidth;
        bandEnd = freqRange(1) + b * bandWidth;
        bandIndices = (freqRangeData >= bandStart & freqRangeData < bandEnd);
        bandEnergies(b) = sum(fftSmoothedRange(bandIndices).^2);
        fprintf('Band %d (%.1f-%.1f Hz): Energy = %.6f\n', b, bandStart, bandEnd, bandEnergies(b));
    end


end
