import socket
import csv
import time
import threading

# Arduino and ESP32 server information
PIEZO_HOST = "172.20.10.12"  # Replace with your piezo server's IP address
PIEZO_PORT = 80
TRANSDUCER_HOST = "172.20.10.2"  # Replace with your transducer's IP address
TRANSDUCER_PORT = 80

# File to save data
output_file = "vibration_data_combined.csv"

def save_to_csv(data):
    """Saves a list of (time, value) tuples to a CSV file."""
    with open(output_file, mode="a", newline="") as file:
        writer = csv.writer(file)
        writer.writerows(data)

def piezo_measurement(duration, start_event, stop_event):
    """Handles data collection from the piezo sensor."""
    try:
        print(f"Connecting to piezo server at {PIEZO_HOST}:{PIEZO_PORT}...")
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as piezo_socket:
            piezo_socket.settimeout(5)  # Set a 5-second timeout for all operations
            piezo_socket.connect((PIEZO_HOST, PIEZO_PORT))
            print("Connected to the piezo server.")

            # Wait for start signal
            start_event.wait()
            piezo_socket.sendall(f"{duration}\n".encode())  # Send recording duration

            # Receive data
            data_buffer = []
            print("Piezo: Receiving data...")
            start_time = time.time()

            while not stop_event.is_set():
                try:
                    data = piezo_socket.recv(1024).decode()
                    if not data:
                        break  # End of data stream

                    # Process received data
                    lines = data.split("\n")
                    for line in lines:
                        if "," in line:  # Ensure it's a valid data line
                            try:
                                timestamp, value = line.split(",")
                                data_buffer.append((int(timestamp), int(value)))
                            except ValueError:
                                pass  # Ignore malformed lines

                except socket.timeout:
                    print("Piezo: No data received. Check the server.")
                    break

            # Save the data
            if data_buffer:
                save_to_csv(data_buffer)
                print(f"Piezo: Data saved to {output_file}")
            else:
                print("Piezo: No data received.")

    except Exception as e:
        print(f"Piezo: An error occurred: {e}")

def run_transducer(start_event):
    """Handles the activation of the transducer."""
    try:
        print(f"Connecting to transducer at {TRANSDUCER_HOST}:{TRANSDUCER_PORT}...")
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as transducer_socket:
            transducer_socket.settimeout(5)  # Set a timeout for the connection
            transducer_socket.connect((TRANSDUCER_HOST, TRANSDUCER_PORT))
            print("Connected to the transducer server.")

            # Wait for start signal
            start_event.wait()

            # Send command to the transducer
            print("Transducer: Activating vibration...")
            transducer_socket.sendall(b"GET / HTTP/1.1\r\nHost: 172.20.10.13\r\n\r\n")
            response = transducer_socket.recv(1024)
            print("Transducer: Response from ESP32:")
            print(response.decode())

    except socket.error as e:
        print(f"Transducer: Socket error: {e}")

# Main function
def main():
    # Events to coordinate actions
    start_event = threading.Event()
    stop_event = threading.Event()

    # User input to start the process
    duration = 10  # Set the recording duration (10 seconds)

    # Create threads
    piezo_thread = threading.Thread(target=piezo_measurement, args=(duration, start_event, stop_event))
    transducer_thread = threading.Thread(target=run_transducer, args=(start_event,))

    # Start threads
    piezo_thread.start()
    transducer_thread.start()

    # Wait for user input
    input("Press 's' to start: ")
    start_event.set()  # Signal both threads to start
    print("Measurement started.")

    # Wait for the duration, then signal piezo to stop
    time.sleep(duration)
    stop_event.set()
    print("Measurement stopped. Saving data...")

    # Wait for threads to finish
    piezo_thread.join()
    transducer_thread.join()
    print("Process complete.")

if __name__ == "__main__":
    main()
